@@ -438,7 +438,7 @@ xdg_mime_get_mime_type_for_data (const void *data,
   if (_caches)
     return _xdg_mime_cache_get_mime_type_for_data (data, len);
 
-  mime_type = _xdg_mime_magic_lookup_data (global_magic, data, len);
+  mime_type = _xdg_mime_magic_lookup_data (global_magic, data, len, NULL, 0);
 
   if (mime_type)
     return mime_type;
@@ -451,12 +451,17 @@ xdg_mime_get_mime_type_for_file (const char  *file_name,
                                  struct stat *statbuf)
 {
   const char *mime_type;
+  /* currently, only a few globs occur twice, and none
+   * more often, so 5 seems plenty.
+   */
+  const char *mime_types[5];
   FILE *file;
   unsigned char *data;
   int max_extent;
   int bytes_read;
   struct stat buf;
   const char *base_name;
+  int n;
 
   if (file_name == NULL)
     return NULL;
@@ -466,13 +471,13 @@ xdg_mime_get_mime_type_for_file (const char  *file_name,
   xdg_mime_init ();
 
   if (_caches)
-    return _xdg_mime_cache_get_mime_type_for_file (file_name);
+    return _xdg_mime_cache_get_mime_type_for_file (file_name, statbuf);
 
   base_name = _xdg_get_base_name (file_name);
-  mime_type = xdg_mime_get_mime_type_from_file_name (base_name);
+  n = _xdg_glob_hash_lookup_file_name (global_hash, base_name, mime_types, 5);
 
-  if (mime_type != XDG_MIME_TYPE_UNKNOWN)
-    return mime_type;
+  if (n == 1)
+    return mime_types[0];
 
   if (!statbuf)
     {
@@ -508,7 +513,8 @@ xdg_mime_get_mime_type_for_file (const char  *file_name,
       return XDG_MIME_TYPE_UNKNOWN;
     }
 
-  mime_type = _xdg_mime_magic_lookup_data (global_magic, data, bytes_read);
+  mime_type = _xdg_mime_magic_lookup_data (global_magic, data, bytes_read,
+					   mime_types, n);
 
   free (data);
   fclose (file);
@@ -529,8 +535,7 @@ xdg_mime_get_mime_type_from_file_name (const char *file_name)
   if (_caches)
     return _xdg_mime_cache_get_mime_type_from_file_name (file_name);
 
-  mime_type = _xdg_glob_hash_lookup_file_name (global_hash, file_name);
-  if (mime_type)
+  if (_xdg_glob_hash_lookup_file_name (global_hash, file_name, &mime_type, 1))
     return mime_type;
   else
     return XDG_MIME_TYPE_UNKNOWN;
